/* Copyright (c) 1999-2001 TTK Ciar / Bill Moyer
 * All rights reserved.                               
 *
 * This code is derived from code developed by TTK Ciar and Bill Moyer.    
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions         
 * are met:                                                                  
 *
 *  1. Redistributions of source code must retain the above copyright        
 *     notice, this list of conditions and the following disclaimer.
 *  2. The name of the author may not be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
// TTK here -- Clarification on above: This license is NOT viral and
// does NOT put you under any obligation to do ANY of the following:
//   ** distribute sources
//   ** contribute modifications
//   ** grant credit or show recognition
//   ** grant any other kind of compensation
// This document may be used and modified for free OR COMMERCIAL 
// development with impunity.  The SOLE intention of this license is 
// to prevent anyone from laying private claim to publicly available 
// versions of the source code and putting any kind of restrictions 
// on its distribution or use.

#ifndef _MISCLIB_C_
#define _MISCLIB_C_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include "misclib.h"

alloc_ft memalloc = (alloc_ft) malloc;
free_ft  memfree  = (free_ft)  free;
sm_ft    panic1   = NULL;
sm_ft    panic2   = NULL;
sm_ft    panic3   = NULL;
int      smalloc_error = 0;
static char *reserve  = NULL;

char *padstr ( char *str, char pad, int w, char *buf ) 
  {
    int len, r, i;
    buf[0] = 0;
    if ( str == NULL ) return ( buf );
    len = strlen ( str );
    if ( len >= w ) 
      { 
        strcpy ( buf, str ); 
        return ( buf );
      }
    r = w - len;
    for ( i = 0; i < r; i++ ) buf[i] = pad;
    strcpy ( &(buf[r]), str );
    return ( buf );
  }

char *padnum ( int num, int w, char *buf ) {
  char numstr[32];
  sprintf ( numstr, "%d", num );
  return padstr ( numstr, '0', w, buf );
  }

char *padhex ( int num, int w, int ucase, char *buf ) {
  char numstr[32];
  if ( ucase ) 
       sprintf ( numstr, "%X", num );
  else sprintf ( numstr, "%x", num );
  return padstr ( numstr, '0', w, buf );
  }

char *strndup( char *str, int limit ) {
  char *retval;
  int   i, k;
  for ( k = 0; ( k < limit ) && ( str[k] != 0 ); k++ );
  retval = memalloc ( k + 1 );
  if ( retval == NULL ) return NULL;
  strncpy ( retval, str, k );
  retval[k] = 0;
  return retval;
  }

char *memdup ( void *p, int n ) {
  char *retval;
  int   i;
  retval = memalloc ( n );
  if ( retval == NULL ) return NULL;
  memcpy ( retval, p, n );
  return retval;
  }

void scrub_free ( char *p, int k )
  {
    int i;
    if ( p == NULL ) return;
    for ( i = 0; i < k; i++ ) p[i] = 0;
    memfree ( p );
  }

void scrub_free_str ( char *p )
  {
    int i;
    if ( p    == NULL ) return;
    if ( p[0] == 0    ) return;
    for ( i = 0, p[0] = 0; p[i+1] != 0; i++ ) p[i] = 0;
    memfree ( p );
  }

/* fdpoll() returns the number of bytes ready to be read from a file. */
int fdpoll ( int fd ) 
  {
    int retval = 0;
    int result;
    result = ioctl ( fd, FIONREAD, &retval );
    if ( result < 0 ) return result;
    return retval;
  }

int smalloc_init ( int res_sz, sm_ft *panicf1, sm_ft *panicf2, sm_ft *panicf3 ) 
  {
    int rsz;
    memalloc = (alloc_ft) smalloc;
    panic1   = (sm_ft)    panicf1;
    panic2   = (sm_ft)    panicf2;
    panic3   = (sm_ft)    panicf3;
    if ( res_sz < 1 ) rsz = SMALLOC_DEFAULT_RESERVE_SIZE; 
    else rsz = res_sz;
    if ( reserve != NULL ) reserve = (char*) malloc ( rsz );
    if ( reserve == NULL ) 
      { 
         smalloc_error = 1;
         return ( -1 );
      }
    smalloc_error = 0;
    return ( 0 );
  }

void *smalloc ( unsigned int n ) {
  char *retval;
  if ( n < 1 ) 
    {
       smalloc_error = 2;
       return NULL;
    }
  retval = (char*) malloc ( n );
  if ( retval == NULL ) 
    {
      if ( reserve == NULL ) 
        { 
           smalloc_error = 3; 
           if ( panic3 != NULL ) panic3(); 
           return ( NULL );
        } else {
          memfree ( reserve ); 
          reserve = NULL;
          retval = (char*) malloc ( n );
          if ( retval == NULL ) 
            { 
              smalloc_error = 4;
              if ( panic2 != NULL ) panic2();
              return ( NULL );
            } 
          if ( panic1 != NULL ) panic1();
          smalloc_error = 5;
          return ( retval );
        } // end of if..else
    } else smalloc_error = 0;
  return retval;
  }

#endif
